pub struct Pair<A, B>(pub A, pub B);Expand description
Sequential composition of formats A and B.
Tuple Fields
0: A1: BTrait Implementations
Sourceimpl<A, B> Consistency for Pair<A, B>where
A: Consistency,
B: Consistency,
impl<A, B> Consistency for Pair<A, B>where
A: Consistency,
B: Consistency,
Sourceopen spec fn consistent(&self, v: Self::Val) -> bool
open spec fn consistent(&self, v: Self::Val) -> bool
{ self.0.consistent(v.0) && self.1.consistent(v.1) }Sourcetype Val = (<A as Consistency>::Val, <B as Consistency>::Val)
type Val = (<A as Consistency>::Val, <B as Consistency>::Val)
The type of values whose consistency is being checked.
Sourceimpl<D1: DepCombinator, D2: DepCombinator> DepCombinator for Pair<D1, D2>
impl<D1: DepCombinator, D2: DepCombinator> DepCombinator for Pair<D1, D2>
Sourceopen spec fn apply(&self, key: Self::Key) -> Self::Body
open spec fn apply(&self, key: Self::Key) -> Self::Body
{ Pair(self.0.apply(key.0), self.1.apply(key.1)) }Sourceopen spec fn recover(&self, value: Self::Val) -> Self::Key
open spec fn recover(&self, value: Self::Val) -> Self::Key
{ (self.0.recover(value.0), self.1.recover(value.1)) }Sourceopen spec fn recover_inv(&self) -> bool
open spec fn recover_inv(&self) -> bool
{ self.0.recover_inv() && self.1.recover_inv() }Sourceproof fn lemma_recover_consistent(&self, key: Self::Key, value: Self::Val)
proof fn lemma_recover_consistent(&self, key: Self::Key, value: Self::Val)
Sourcetype Key = (<D1 as DepCombinator>::Key, <D2 as DepCombinator>::Key)
type Key = (<D1 as DepCombinator>::Key, <D2 as DepCombinator>::Key)
The type of keys parsed by the head combinator/to be recovered during serialization.
Sourcetype Val = (<D1 as DepCombinator>::Val, <D2 as DepCombinator>::Val)
type Val = (<D1 as DepCombinator>::Val, <D2 as DepCombinator>::Val)
The type of values consumed/produced by the body combinator.
Sourcetype Body = Pair<<D1 as DepCombinator>::Body, <D2 as DepCombinator>::Body>
type Body = Pair<<D1 as DepCombinator>::Body, <D2 as DepCombinator>::Body>
The type of the body combinator produced by
apply.Sourceimpl<A, B, TA, TB> DerOrd<(TA, TB)> for Pair<A, B>
impl<A, B, TA, TB> DerOrd<(TA, TB)> for Pair<A, B>
Sourceproof fn lemma_der_serialize_len(&self, value: (TA::V, TB::V))
proof fn lemma_der_serialize_len(&self, value: (TA::V, TB::V))
Sourceopen spec fn der_remaining(
&self,
value: (TA::V, TB::V),
state: PairDerState<A::State, B::State>,
) -> Seq<u8>
open spec fn der_remaining( &self, value: (TA::V, TB::V), state: PairDerState<A::State, B::State>, ) -> Seq<u8>
{
if state.in_left {
self.0.der_remaining(value.0, state.left)
+ self.1.der_remaining(value.1, state.right)
} else {
self.1.der_remaining(value.1, state.right)
}
}Sourceopen spec fn der_state_valid(
&self,
value: (TA::V, TB::V),
state: PairDerState<A::State, B::State>,
) -> bool
open spec fn der_state_valid( &self, value: (TA::V, TB::V), state: PairDerState<A::State, B::State>, ) -> bool
{
&&& self.0.der_state_valid(value.0, state.left)
&&& self.1.der_state_valid(value.1, state.right)
&&& !state.in_left ==> self.0.der_remaining(value.0, state.left).len() == 0
}Sourceimpl<A, B> EquivSerializers for Pair<A, B>where
A: EquivSerializersGeneral,
B: EquivSerializers,
impl<A, B> EquivSerializers for Pair<A, B>where
A: EquivSerializersGeneral,
B: EquivSerializers,
Sourceimpl<A, B> EquivSerializersGeneral for Pair<A, B>where
A: EquivSerializersGeneral,
B: EquivSerializersGeneral,
impl<A, B> EquivSerializersGeneral for Pair<A, B>where
A: EquivSerializersGeneral,
B: EquivSerializersGeneral,
Sourceopen spec fn equiv_general_inv(&self) -> bool
open spec fn equiv_general_inv(&self) -> bool
{
&&& self.0.equiv_general_inv()
&&& self.1.equiv_general_inv()
}Sourceproof fn lemma_serialize_equiv(&self, v: Self::SVal, obuf: Seq<u8>)
proof fn lemma_serialize_equiv(&self, v: Self::SVal, obuf: Seq<u8>)
Sourceimpl<A, B> GoodSerializer for Pair<A, B>where
A: GoodSerializer,
B: GoodSerializer,
impl<A, B> GoodSerializer for Pair<A, B>where
A: GoodSerializer,
B: GoodSerializer,
Sourceopen spec fn serialize_inv(&self) -> bool
open spec fn serialize_inv(&self) -> bool
{
&&& self.0.serialize_inv()
&&& self.1.serialize_inv()
}Sourceproof fn lemma_serialize_len(&self, v: Self::SVal)
proof fn lemma_serialize_len(&self, v: Self::SVal)
Sourceimpl<Left: HasAsn1Start, Right: SpecParser> HasAsn1Start for Pair<Left, Right>
A required pair starts wherever its required left component starts.
impl<Left: HasAsn1Start, Right: SpecParser> HasAsn1Start for Pair<Left, Right>
A required pair starts wherever its required left component starts.
Sourceopen spec fn asn1_start(&self) -> Asn1StartDomain
open spec fn asn1_start(&self) -> Asn1StartDomain
{ self.0.asn1_start() }Sourceproof fn lemma_parse_implies_asn1_start(&self, input: Seq<u8>)
proof fn lemma_parse_implies_asn1_start(&self, input: Seq<u8>)
Sourceimpl<A: MinMaxByteLen, B: MinMaxByteLen> MinMaxByteLen for Pair<A, B>
impl<A: MinMaxByteLen, B: MinMaxByteLen> MinMaxByteLen for Pair<A, B>
Sourceimpl<A: NoLookAhead, B: NoLookAhead> NoLookAhead for Pair<A, B>
impl<A: NoLookAhead, B: NoLookAhead> NoLookAhead for Pair<A, B>
Sourceopen spec fn no_lookahead_inv(&self) -> bool
open spec fn no_lookahead_inv(&self) -> bool
{
&&& self.0.no_lookahead_inv()
&&& self.1.no_lookahead_inv()
}Sourceproof fn lemma_no_lookahead(&self, i1: Seq<u8>, i2: Seq<u8>)
proof fn lemma_no_lookahead(&self, i1: Seq<u8>, i2: Seq<u8>)
Sourceimpl<A: NonMalleable, B: NonMalleable> NonMalleable for Pair<A, B>
impl<A: NonMalleable, B: NonMalleable> NonMalleable for Pair<A, B>
Sourceopen spec fn nonmal_inv(&self) -> bool
open spec fn nonmal_inv(&self) -> bool
{
&&& self.0.nonmal_inv()
&&& self.1.nonmal_inv()
&&& self.0.safe_inv()
&&& self.1.safe_inv()
}Sourceproof fn lemma_parse_non_malleable(&self, buf1: Seq<u8>, buf2: Seq<u8>)
proof fn lemma_parse_non_malleable(&self, buf1: Seq<u8>, buf2: Seq<u8>)
Sourceimpl<A, B> NonTailFmt for Pair<A, B>where
A: NonTailFmt,
B: NonTailFmt,
impl<A, B> NonTailFmt for Pair<A, B>where
A: NonTailFmt,
B: NonTailFmt,
Sourceopen spec fn serialize_dps_inv(&self) -> bool
open spec fn serialize_dps_inv(&self) -> bool
{
&&& self.0.serialize_dps_inv()
&&& self.1.serialize_dps_inv()
}Sourceproof fn lemma_serialize_dps_prepend(&self, v: Self::SValue, obuf: Seq<u8>)
proof fn lemma_serialize_dps_prepend(&self, v: Self::SValue, obuf: Seq<u8>)
Sourceproof fn lemma_serialize_dps_len(&self, v: Self::SValue, obuf: Seq<u8>)
proof fn lemma_serialize_dps_len(&self, v: Self::SValue, obuf: Seq<u8>)
Sourceimpl<A: Productive, B: Productive> Productive for Pair<A, B>
impl<A: Productive, B: Productive> Productive for Pair<A, B>
Sourceopen spec fn productive_inv(&self) -> bool
open spec fn productive_inv(&self) -> bool
{
||| self.0.productive_inv()
||| self.1.productive_inv()
}Sourceproof fn lemma_productive(&self, s: Seq<u8>)
proof fn lemma_productive(&self, s: Seq<u8>)
Sourceimpl<A: SPRoundTripDps + NonTailFmt, B: SPRoundTripDps> SPRoundTripDps for Pair<A, B>
impl<A: SPRoundTripDps + NonTailFmt, B: SPRoundTripDps> SPRoundTripDps for Pair<A, B>
Sourceopen spec fn unambiguous(&self) -> bool
open spec fn unambiguous(&self) -> bool
{
&&& self.0.serialize_dps_inv()
&&& self.0.unambiguous()
&&& self.1.unambiguous()
}Sourceproof fn theorem_serialize_dps_parse_roundtrip(&self, v: Self::T, obuf: Seq<u8>)
proof fn theorem_serialize_dps_parse_roundtrip(&self, v: Self::T, obuf: Seq<u8>)
Sourceimpl<A, B> SafeParser for Pair<A, B>where
A: SafeParser,
B: SafeParser,
impl<A, B> SafeParser for Pair<A, B>where
A: SafeParser,
B: SafeParser,
Sourceimpl<Output: OutputBuf, A, B, TA, TB> Serializer<Output, (TA, TB)> for Pair<A, B>where
TA: DeepView,
TB: DeepView,
A: Serializer<Output, TA>,
B: Serializer<Output, TB>,
impl<Output: OutputBuf, A, B, TA, TB> Serializer<Output, (TA, TB)> for Pair<A, B>where
TA: DeepView,
TB: DeepView,
A: Serializer<Output, TA>,
B: Serializer<Output, TB>,
Sourceexec fn serialize_into(&self, v: &(TA, TB), obuf: &mut Output)
exec fn serialize_into(&self, v: &(TA, TB), obuf: &mut Output)
Sourceimpl<A, B> SoundParser for Pair<A, B>where
A: SoundParser,
B: SoundParser,
impl<A, B> SoundParser for Pair<A, B>where
A: SoundParser,
B: SoundParser,
Sourceimpl<A: SpecByteLen, B: SpecByteLen> SpecByteLen for Pair<A, B>
impl<A: SpecByteLen, B: SpecByteLen> SpecByteLen for Pair<A, B>
Sourceimpl<A, B> SpecParser for Pair<A, B>where
A: SpecParser,
B: SpecParser,
impl<A, B> SpecParser for Pair<A, B>where
A: SpecParser,
B: SpecParser,
Sourceopen spec fn spec_parse(&self, ibuf: Seq<u8>) -> Option<(int, Self::PVal)>
open spec fn spec_parse(&self, ibuf: Seq<u8>) -> Option<(int, Self::PVal)>
{
match self.0.spec_parse(ibuf) {
Some((n1, v1)) => {
match self.1.spec_parse(ibuf.skip(n1)) {
Some((n2, v2)) => Some((n1 + n2, (v1, v2))),
None => None,
}
}
None => None,
}
}Sourcetype PVal = (<A as SpecParser>::PVal, <B as SpecParser>::PVal)
type PVal = (<A as SpecParser>::PVal, <B as SpecParser>::PVal)
The type of parsed values.
Sourceimpl<A, B> SpecSerializer for Pair<A, B>where
A: SpecSerializer,
B: SpecSerializer,
impl<A, B> SpecSerializer for Pair<A, B>where
A: SpecSerializer,
B: SpecSerializer,
Sourceopen spec fn spec_serialize(&self, v: Self::SVal) -> Seq<u8>
open spec fn spec_serialize(&self, v: Self::SVal) -> Seq<u8>
{ self.0.spec_serialize(v.0) + self.1.spec_serialize(v.1) }Sourcetype SVal = (<A as SpecSerializer>::SVal, <B as SpecSerializer>::SVal)
type SVal = (<A as SpecSerializer>::SVal, <B as SpecSerializer>::SVal)
The type of values to be serialized.
Sourceimpl<A, B> SpecSerializerDps for Pair<A, B>where
A: SpecSerializerDps,
B: SpecSerializerDps,
impl<A, B> SpecSerializerDps for Pair<A, B>where
A: SpecSerializerDps,
B: SpecSerializerDps,
Sourceopen spec fn spec_serialize_dps(&self, v: Self::SValue, obuf: Seq<u8>) -> Seq<u8>
open spec fn spec_serialize_dps(&self, v: Self::SValue, obuf: Seq<u8>) -> Seq<u8>
{ self.0.spec_serialize_dps(v.0, self.1.spec_serialize_dps(v.1, obuf)) }Sourcetype SValue = (<A as SpecSerializerDps>::SValue, <B as SpecSerializerDps>::SValue)
type SValue = (<A as SpecSerializerDps>::SValue, <B as SpecSerializerDps>::SValue)
The type of values to be serialized.
Sourceimpl<A: StaticByteLen, B: StaticByteLen> StaticByteLen for Pair<A, B>
impl<A: StaticByteLen, B: StaticByteLen> StaticByteLen for Pair<A, B>
Sourceopen spec fn static_byte_len() -> nat
open spec fn static_byte_len() -> nat
{ A::static_byte_len() + B::static_byte_len() }Sourceproof fn lemma_static_len_matches_byte_len(&self, v: Self::T)
proof fn lemma_static_len_matches_byte_len(&self, v: Self::T)
Sourceimpl<A: ValueByteLen, B: ValueByteLen> ValueByteLen for Pair<A, B>
impl<A: ValueByteLen, B: ValueByteLen> ValueByteLen for Pair<A, B>
Sourceopen spec fn value_byte_len(v: Self::T) -> nat
open spec fn value_byte_len(v: Self::T) -> nat
{ A::value_byte_len(v.0) + B::value_byte_len(v.1) }Sourceproof fn lemma_value_len_matches_byte_len(&self, v: Self::T)
proof fn lemma_value_len_matches_byte_len(&self, v: Self::T)
impl<A: Copy, B: Copy> Copy for Pair<A, B>