pub struct Bind<A, B>(pub A, pub B);Expand description
Sequential composition of formats A and B, where B may depend on the value of A.
Parsing semantics: parses A to get a key, then parses B(key) to get the body value,
and returns (key, value).
During serialization, the caller must provide both the key and value.
Note on usage
Prefer super::Implicit when the key should be recovered from the body value instead of
being carried explicitly through the value type.
Tuple Fields
0: A1: BTrait Implementations
Sourceimpl<A, B, STA, STB> ByteLen<(STA, STB)> for Bind<A, B>
impl<A, B, STA, STB> ByteLen<(STA, STB)> for Bind<A, B>
Sourceimpl<A, B> Consistency for Bind<A, B>
impl<A, B> Consistency for Bind<A, B>
Sourceopen spec fn consistent(&self, value: Self::Val) -> bool
open spec fn consistent(&self, value: Self::Val) -> bool
{
let (key, val) = value;
self.0.consistent(key) && self.1.spec_map(key).consistent(val)
}Sourcetype Val = (<A as Consistency>::Val, <<B as SpecMap>::Output as Consistency>::Val)
type Val = (<A as Consistency>::Val, <<B as SpecMap>::Output as Consistency>::Val)
The type of values whose consistency is being checked.
Sourceimpl<A, B> EquivSerializers for Bind<A, B>
impl<A, B> EquivSerializers for Bind<A, B>
Sourceimpl<A, B> EquivSerializersGeneral for Bind<A, B>
impl<A, B> EquivSerializersGeneral for Bind<A, B>
Sourceopen spec fn equiv_general_inv(&self) -> bool
open spec fn equiv_general_inv(&self) -> bool
{
&&& self.0.equiv_general_inv()
&&& forall |key: A::SVal| #[trigger] self.1.spec_map(key).equiv_general_inv()
}Sourceproof fn lemma_serialize_equiv(&self, value: Self::SVal, obuf: Seq<u8>)
proof fn lemma_serialize_equiv(&self, value: Self::SVal, obuf: Seq<u8>)
Sourceimpl<A, B> GoodSerializer for Bind<A, B>
impl<A, B> GoodSerializer for Bind<A, B>
Sourceopen spec fn serialize_inv(&self) -> bool
open spec fn serialize_inv(&self) -> bool
{
&&& self.0.serialize_inv()
&&& forall |key: A::SVal| #[trigger] self.1.spec_map(key).serialize_inv()
}Sourceproof fn lemma_serialize_len(&self, value: Self::SVal)
proof fn lemma_serialize_len(&self, value: Self::SVal)
Sourceimpl<A, B> NoLookAhead for Bind<A, B>
impl<A, B> NoLookAhead for Bind<A, B>
Sourceopen spec fn no_lookahead_inv(&self) -> bool
open spec fn no_lookahead_inv(&self) -> bool
{
&&& self.0.no_lookahead_inv()
&&& forall |key: A::PVal| #[trigger] self.1.spec_map(key).no_lookahead_inv()
}Sourceproof fn lemma_no_lookahead(&self, i1: Seq<u8>, i2: Seq<u8>)
proof fn lemma_no_lookahead(&self, i1: Seq<u8>, i2: Seq<u8>)
Sourceimpl<A, B> NonMalleable for Bind<A, B>
impl<A, B> NonMalleable for Bind<A, B>
Sourceopen spec fn nonmal_inv(&self) -> bool
open spec fn nonmal_inv(&self) -> bool
{
&&& self.0.nonmal_inv()
&&& self.0.safe_inv()
&&& forall |key: A::PVal| #[trigger] self.1.spec_map(key).nonmal_inv()
&&& forall |key: A::PVal| #[trigger] self.1.spec_map(key).safe_inv()
}Sourceproof fn lemma_parse_non_malleable(&self, buf1: Seq<u8>, buf2: Seq<u8>)
proof fn lemma_parse_non_malleable(&self, buf1: Seq<u8>, buf2: Seq<u8>)
Sourceimpl<A, B> NonTailFmt for Bind<A, B>
impl<A, B> NonTailFmt for Bind<A, B>
Sourceopen spec fn serialize_dps_inv(&self) -> bool
open spec fn serialize_dps_inv(&self) -> bool
{
&&& self.0.serialize_dps_inv()
&&& forall |key: A::SValue| #[trigger] self.1.spec_map(key).serialize_dps_inv()
}Sourceproof fn lemma_serialize_dps_prepend(&self, value: Self::SValue, obuf: Seq<u8>)
proof fn lemma_serialize_dps_prepend(&self, value: Self::SValue, obuf: Seq<u8>)
Sourceproof fn lemma_serialize_dps_len(&self, value: Self::SValue, obuf: Seq<u8>)
proof fn lemma_serialize_dps_len(&self, value: Self::SValue, obuf: Seq<u8>)
Sourceimpl<I, A, B> Parser<I> for Bind<A, B>where
I: InputBuf,
A: Parser<I> + SafeParser,
B::O: Parser<I> + SafeParser,
B: MapRef<A::PT, Input = A::PVal>,
impl<I, A, B> Parser<I> for Bind<A, B>where
I: InputBuf,
A: Parser<I> + SafeParser,
B::O: Parser<I> + SafeParser,
B: MapRef<A::PT, Input = A::PVal>,
Sourceimpl<A, B, STA, STB> Prepare<(STA, STB)> for Bind<A, B>
impl<A, B, STA, STB> Prepare<(STA, STB)> for Bind<A, B>
Sourceimpl<A, B> Productive for Bind<A, B>
impl<A, B> Productive for Bind<A, B>
Sourceopen spec fn productive_inv(&self) -> bool
open spec fn productive_inv(&self) -> bool
{
||| self.0.productive_inv()
||| forall |key: A::PVal| #[trigger] self.1.spec_map(key).productive_inv()
}Sourceproof fn lemma_productive(&self, s: Seq<u8>)
proof fn lemma_productive(&self, s: Seq<u8>)
Sourceimpl<A, B> SPRoundTripDps for Bind<A, B>
impl<A, B> SPRoundTripDps for Bind<A, B>
Sourceopen spec fn unambiguous(&self) -> bool
open spec fn unambiguous(&self) -> bool
{
&&& self.0.serialize_dps_inv()
&&& self.0.unambiguous()
&&& forall |key: A::T| #[trigger] self.1.spec_map(key).unambiguous()
}Sourceproof fn theorem_serialize_dps_parse_roundtrip(&self, value: Self::T, obuf: Seq<u8>)
proof fn theorem_serialize_dps_parse_roundtrip(&self, value: Self::T, obuf: Seq<u8>)
Sourceimpl<A, B> SafeParser for Bind<A, B>
impl<A, B> SafeParser for Bind<A, B>
Sourceimpl<Output: OutputBuf, A, B, TA, TB> Serializer<Output, (TA, TB)> for Bind<A, B>where
TA: DeepView,
TB: DeepView,
A: Serializer<Output, TA>,
B::O: Serializer<Output, TB>,
B: MapRef<TA, Input = TA::V>,
impl<Output: OutputBuf, A, B, TA, TB> Serializer<Output, (TA, TB)> for Bind<A, B>where
TA: DeepView,
TB: DeepView,
A: Serializer<Output, TA>,
B::O: Serializer<Output, TB>,
B: MapRef<TA, Input = TA::V>,
Sourceopen spec fn exec_inv(&self) -> bool
open spec fn exec_inv(&self) -> bool
{
&&& self.0.exec_inv()
&&& forall |pb: B::O| #[trigger] pb.exec_inv()
}Sourceexec fn serialize_into(&self, v: &(TA, TB), obuf: &mut Output)
exec fn serialize_into(&self, v: &(TA, TB), obuf: &mut Output)
Sourceimpl<A, B> SoundParser for Bind<A, B>
impl<A, B> SoundParser for Bind<A, B>
Sourceimpl<A, B> SpecByteLen for Bind<A, B>
impl<A, B> SpecByteLen for Bind<A, B>
Sourceimpl<A, B> SpecParser for Bind<A, B>
impl<A, B> SpecParser for Bind<A, B>
Sourceopen spec fn spec_parse(&self, ibuf: Seq<u8>) -> Option<(int, Self::PVal)>
open spec fn spec_parse(&self, ibuf: Seq<u8>) -> Option<(int, Self::PVal)>
{
match self.0.spec_parse(ibuf) {
Some((n1, key)) => {
let next = self.1.spec_map(key);
match next.spec_parse(ibuf.skip(n1)) {
Some((n2, val)) => Some((n1 + n2, (key, val))),
None => None,
}
}
None => None,
}
}Sourcetype PVal = (<A as SpecParser>::PVal, <<B as SpecMap>::Output as SpecParser>::PVal)
type PVal = (<A as SpecParser>::PVal, <<B as SpecMap>::Output as SpecParser>::PVal)
The type of parsed values.
Sourceimpl<A, B> SpecSerializer for Bind<A, B>
impl<A, B> SpecSerializer for Bind<A, B>
Sourceopen spec fn spec_serialize(&self, v: Self::SVal) -> Seq<u8>
open spec fn spec_serialize(&self, v: Self::SVal) -> Seq<u8>
{
let (key, val) = v;
let next = self.1.spec_map(key);
self.0.spec_serialize(key) + next.spec_serialize(val)
}Sourcetype SVal = (<A as SpecSerializer>::SVal, <<B as SpecMap>::Output as SpecSerializer>::SVal)
type SVal = (<A as SpecSerializer>::SVal, <<B as SpecMap>::Output as SpecSerializer>::SVal)
The type of values to be serialized.
Sourceimpl<A, B> SpecSerializerDps for Bind<A, B>
impl<A, B> SpecSerializerDps for Bind<A, B>
Sourceopen spec fn spec_serialize_dps(&self, value: Self::SValue, obuf: Seq<u8>) -> Seq<u8>
open spec fn spec_serialize_dps(&self, value: Self::SValue, obuf: Seq<u8>) -> Seq<u8>
{
let (key, val) = value;
let next = self.1.spec_map(key);
self.0.spec_serialize_dps(key, next.spec_serialize_dps(val, obuf))
}Sourcetype SValue = (<A as SpecSerializerDps>::SValue, <<B as SpecMap>::Output as SpecSerializerDps>::SValue)
type SValue = (<A as SpecSerializerDps>::SValue, <<B as SpecMap>::Output as SpecSerializerDps>::SValue)
The type of values to be serialized.
Sourceimpl<A, B> StaticByteLen for Bind<A, B>
impl<A, B> StaticByteLen for Bind<A, B>
Sourceopen spec fn static_byte_len() -> nat
open spec fn static_byte_len() -> nat
{ A::static_byte_len() + <B::Output as StaticByteLen>::static_byte_len() }Sourceproof fn lemma_static_len_matches_byte_len(&self, value: Self::T)
proof fn lemma_static_len_matches_byte_len(&self, value: Self::T)
Sourceimpl<A, B> ValueByteLen for Bind<A, B>
impl<A, B> ValueByteLen for Bind<A, B>
Sourceopen spec fn value_byte_len(value: Self::T) -> nat
open spec fn value_byte_len(value: Self::T) -> nat
{ A::value_byte_len(value.0) + <B::Output as ValueByteLen>::value_byte_len(value.1) }Sourceproof fn lemma_value_len_matches_byte_len(&self, value: Self::T)
proof fn lemma_value_len_matches_byte_len(&self, value: Self::T)
impl<A: Copy, B: Copy> Copy for Bind<A, B>